THIS IS UNUSED
assert False


cdef int count_liberties(board* b, int c):
    assert MAX_LIBERTY_COUNT <= GROUP_REFILL_LIBS
    return board_group_info(b, group_at(b, c)).libs


cdef int MAX_LIBERTY_COUNT = 4


# "player" is either S_BLACK or S_WHITE
# "move" is a Pachi coord_t
cdef class Transition:
    cdef public int color
    cdef public PyPachiBoard state
    cdef public int move
    cdef public PyPachiBoard next_state
    def __init__(self, int color, PyPachiBoard state, int move, PyPachiBoard next_state):
        assert color == 0 or color == 1
        self.color = color
        self.state = state
        self.move = move
        self.next_state = next_state
    def __richcmp__(Transition self, Transition other, int op):
        if op != 2 and op != 3: return NotImplemented
        cdef bint eq = self.color == other.color and self.state == other.state and self.move == other.move and self.next_state == other.next_state
        return eq if op == 2 else not eq



cdef class PyRolloutResult:
    cdef RolloutResult r
    def __cinit__(self, RolloutResult r_):
        self.r = r_
    property black_final_score:
        def __get__(self):
            return self.r.black_final_score
    property white_final_score:
        def __get__(self):
            return self.r.white_final_score
    property black_resigned:
        def __get__(self):
            return self.r.black_resigned
    property white_resigned:
        def __get__(self):
            return self.r.white_resigned
    property winner:
        def __get__(self):
            return color2player(Winner(self.r))

cdef object _opponent_policy = None
cdef coord_t _opponent_policy_wrapper(PachiBoardPtr b, stone color):
    cdef int action = _opponent_policy(wrap_board(b), color2player(color))
    return action2coord(b.get().pachiboard(), action)

cdef object collect_transitions(const vector[PachiBoardPtr]& bhist, const vector[move]& mhist):
    transitions = []
    cdef int i, a
    for i in range(mhist.size()):
        a = coord2action(bhist[i].get().pachiboard(), mhist[i].coord)
        if a == RESIGN_ACTION_ID: break # don't expose resigns
        transitions.append(Transition(
            player=color2player(mhist[i].color),
            state=wrap_board(bhist[i]),
            action=a,
            next_state=wrap_board(bhist[i+1])
        ))
    return transitions

def rollout_against_pachi(
        PyPachiBoard init_b, int starting_player,
        int pachi_player, string pachi_engine_type, string pachi_engine_arg, string pachi_timestr,
        object opponent_policy_func):
    cdef vector[PachiBoardPtr] bhist
    cdef vector[move] mhist
    global _opponent_policy; _opponent_policy = opponent_policy_func
    cdef PyRolloutResult result = PyRolloutResult(RolloutAgainstPachi(
        init_b._bptr,
        player2color(starting_player),
        player2color(pachi_player), pachi_engine_type, pachi_engine_arg, pachi_timestr,
        _opponent_policy_wrapper,
        &bhist, &mhist))
    assert bhist.size() == mhist.size() + 1
    return result, collect_transitions(bhist, mhist)



### Serialization of transition lists ###
def save_games(games):
    cdef int out_size = 0
    for transitions in games:
        out_size += 2 + 2*len(transitions)

    cdef np.ndarray[np.uint8_t,ndim=1] out = np.empty(out_size, np.uint8)
    cdef PyPachiBoard init_state
    cdef Transition t
    cdef unsigned int pos = 0
    for transitions in games:
        init_state = transitions[0].state
        assert init_state.empty, "only supports games starting from the empty state"
        assert 0 <= init_state.size() < 256
        assert 0 <= len(transitions) < 256
        out[pos] = init_state.size()
        out[pos+1] = len(transitions)
        pos += 2
        for t in transitions:
            assert 0 <= t.player < 256
            assert 0 <= t.action < 256
            out[pos] = t.player
            out[pos+1] = t.action
            pos += 2
    assert pos == out_size

    return out

def reconstruct_games(np.uint8_t[:] data):
    cdef int i, gamelen, boardsize, pos = 0
    cdef vector[np.uint8_t] players, actions
    cdef PyPachiBoard b, b2

    games = []
    while pos < len(data):
        # read board size, game length, and actions
        boardsize = data[pos]
        gamelen = data[pos+1]
        players.clear(); actions.clear()
        for i in range(gamelen):
            players.push_back(data[pos+2 + 2*i])
            actions.push_back(data[pos+2 + 2*i+1])
        pos += 2 + 2*gamelen

        # replay
        transitions = []
        b = CreateBoard(boardsize)
        for i in range(gamelen):
            b2 = b.play(actions[i], players[i])
            transitions.append(Transition(
                player=players[i], state=b, action=actions[i], next_state=b2))
            b = b2
        games.append(transitions)

        if len(games) % 1000 == 0:
            print '[%.2f%%]' % (100.*float(pos)/len(data))

    assert pos == len(data)
    return games

## Deserialization from SGF ###
cdef move move_from_gtp(board* b, char* colorstr, char* coordstr):
    cdef move m
    m.color = str2stone(colorstr)
    cdef coord_t *c = str2coord(coordstr, board_size(b))
    m.coord = deref(c)
    coord_done(c)
    return m
cdef PyPachiBoard play_move(PyPachiBoard b, move m):
    return wrap_board(Play(b._bptr, m))
cdef PyPachiBoard play_gtp(PyPachiBoard b, char* colorstr, char* coordstr):
   return play_move(b, move_from_gtp(b._b.pachiboard(), colorstr, coordstr))

from pygo.sgflib import SGFParser
def ReplaySGF(const char* filename):
    # Adapted from sgf2gtp.py in pachi

    with open(filename, 'r') as f:
        sgfdata = f.read()
    col = SGFParser(sgfdata).parse()
    assert len(col) == 1

    gametree = col[0]

    class UnknownNode(Exception):
        pass

    def get_atr(node, atr):
        try:
            return node.data[atr].data[0]
        except KeyError:
            return None

    def get_setup(node, atr):
        try:
            return node.data[atr].data[:]
        except KeyError:
            return None

    def col2num(column, board_size):
        a, o, z = map(ord, ['a', column, 'z'])
        if a <= o <= z:
            return a + board_size - o
        raise Exception( "Wrong column character: '%s'"%(column,) )

    def is_pass_move(coord, board_size):
        # the pass move is represented either by [] ( = empty coord )
        # OR by [tt] (for boards <= 19 only)
        return len(coord) == 0 or ( board_size <= 19 and coord == 'tt' )

    def record_step(PyPachiBoard b, object coord):
        cdef move m
        if is_pass_move(coord, bsize):
            m = move_from_gtp(b._b.pachiboard(), player_next, 'pass')
        else:
            x, y = coord
            # The reason for this incredibly weird thing is that
            # the GTP protocol excludes `i` in the coordinates
            # (do you see any purpose in this??)
            if x >= 'i':
                x = chr(ord(x)+1)
            y = str(col2num(y, bsize))
            m = move_from_gtp(b._b.pachiboard(), player_next, x+y)

        cdef PyPachiBoard next_b = play_move(b, m)
        return Transition(
            m.color,
            b,
            m.coord,
            next_b
        )

    # cursor for tree traversal
    c = gametree.cursor()
    # first node is the header
    header = c.node

    handicap = get_atr(header, 'HA')
    bsize = int(get_atr(header, 'SZ') or 19)
    # assert bsize == 19

    komi = get_atr(header, 'KM')
    player_next, player_other = 'B', 'W'
    setup_black = get_setup(header, 'AB')
    setup_white = get_setup(header, 'AW')
    ruleset = get_atr(header, 'RU')

    cdef PyPachiBoard b = CreateBoard(bsize)

    # if komi:
    #     b._pachi_board.komi = float(komi)
    if handicap and handicap != '0':
        #board_handicap(b._pachi_board, int(handicap), NULL)
        # The dataset seems to have explicit AB/AW for handicap stones
        # so don't use Pachi's add-handicap method here
        player_next, player_other = player_other, player_next
    if setup_black:
        for item in setup_black:
            x, y = item
            if x >= 'i':
                x = chr(ord(x)+1)
            y = str(col2num(y, bsize))
            b = play_gtp(b, 'B', x+y)
    if setup_white:
        for item in setup_white:
            x, y = item
            if x >= 'i':
                x = chr(ord(x)+1)
            y = str(col2num(y, bsize))
            b = play_gtp(b, 'W', x+y)
    if ruleset:
        if ruleset.lower() == 'nz':
            # board_set_rules only takes new_zealand
            ruleset = 'new_zealand'
        if not board_set_rules(b._b.pachiboard(), ruleset):
            print 'Unrecognized ruleset %s for %s, defaulting to Pachi default' % (ruleset, filename)

    # walk the game tree forward
    game_transitions = []
    while True:
        # sgf2gtp.pl ignores n = 0
        if c.atEnd:
            break
        c.next()

        coord = get_atr(c.node, player_next)
        if coord != None:
            game_transitions.append(record_step(b, coord))
            b = game_transitions[-1].next_state
        else:
            # MAYBE white started?
            # or one of the players plays two time in a row
            player_next, player_other = player_other, player_next
            coord = get_atr(c.node, player_next)
            if coord != None:
                game_transitions.append(record_step(b, coord))
                b = game_transitions[-1].next_state
            elif len(c.node) != 0:
                # TODO handle weird sgf files better
                raise UnknownNode

        player_next, player_other = player_other, player_next
    return game_transitions



#! /usr/bin/env python

import sys
import argparse
import re

from sgflib import SGFParser

DEBUG = False

parser = argparse.ArgumentParser( formatter_class=argparse.RawDescriptionHelpFormatter,
description="""
This script converts SGF files to GTP format so that you can feed them
to Pachi, insert genmove at the right places etc. Might not work on
obscure SGF files.

When called with FILENAMES argument, it will create according output
files with .gtp extension instead of .sgf, unless --no-gtp option is
specified.

Otherwise the games are read from standard input. Note that the stdin
in this case is read in at once, so for very large collections, it is
better to run this script separately for each sgf file.

example:
    cat *.sgf | %s -g -n 5
"""%(sys.argv[0]))

parser.add_argument('FILENAMES', help='List of sgf games to process.', nargs='*', default=[])
parser.add_argument('-g', help='Automatically append genmove command for the other color.', action='store_true')
parser.add_argument('-n', help='Output at most first MOVENUM moves.', metavar='MOVENUM', type=int, default=10**10)
parser.add_argument('--stdout-only', help='Do not create the .gtp files from FILENAMES, print everything to stdout.', action='store_true')
args = vars(parser.parse_args())

class UnknownNode(Exception):
    pass

def get_atr(node, atr):
    try:
        return node.data[atr].data[0]
    except KeyError:
        return None

def get_setup(node, atr):
    try:
        return node.data[atr].data[:]
    except KeyError:
        return None

def col2num(column, board_size):
    a, o, z = map(ord, ['a', column, 'z'])
    if a <= o <= z:
        return a + board_size - o
    raise Exception( "Wrong column character: '%s'"%(column,) )

def is_pass_move(coord, board_size):
    # the pass move is represented either by [] ( = empty coord )
    # OR by [tt] (for boards <= 19 only)
    return len(coord) == 0 or ( board_size <= 19 and coord == 'tt' )

def process_gametree(gametree, fout):
    # cursor for tree traversal
    c = gametree.cursor()
    # first node is the header
    header = c.node

    handicap = get_atr(header, 'HA')
    board_size = int(get_atr(header, 'SZ') or 19)
    komi = get_atr(header, 'KM')
    player_next, player_other = "B", "W"
    setup_black = get_setup(header, 'AB')
    setup_white = get_setup(header, 'AW')

    print >>fout, "boardsize", board_size
    print >>fout, "clear_board"
    if komi:
        print >>fout, "komi", komi
    if handicap and handicap != '0':
        print >>fout, "fixed_handicap", handicap
        player_next, player_other = player_other, player_next
    if setup_black:
	for item in setup_black:
	    x, y = item
	    if x >= 'i':
		x = chr(ord(x)+1)
	    y = str(col2num(y, board_size))
	    print >>fout, "play B", x+y
    if setup_white:
	for item in setup_white:
	    x, y = item
	    if x >= 'i':
		x = chr(ord(x)+1)
	    y = str(col2num(y, board_size))
	    print >>fout, "play W", x+y

    def print_game_step(coord):
        if is_pass_move(coord, board_size):
            print >>fout, "play", player_next, "pass"
        else:
            x, y = coord
            # The reason for this incredibly weird thing is that
            # the GTP protocol excludes `i` in the coordinates
            # (do you see any purpose in this??)
            if x >= 'i':
                x = chr(ord(x)+1)
            y = str(col2num(y, board_size))
            print >>fout, "play", player_next, x+y

    movenum = 0
    # walk the game tree forward
    while 1:
        # sgf2gtp.pl ignores n = 0
        if c.atEnd or (args['n'] and movenum >= args['n']):
            break
        c.next()
        movenum += 1

        coord = get_atr(c.node, player_next)
        if coord != None:
            print_game_step(coord)
        else:
            # MAYBE white started?
            # or one of the players plays two time in a row
            player_next, player_other = player_other, player_next
            coord = get_atr(c.node, player_next)
            if coord != None:
                print_game_step(coord)
            else:
                # TODO handle weird sgf files better
                raise UnknownNode

        player_next, player_other = player_other, player_next

    if args['g']:
        print >>fout, "genmove", player_next

def process_sgf_file(fin, fout):
    sgfdata = fin.read()
    col = SGFParser(sgfdata).parse()

    for gametree in col:
        try:
            process_gametree(gametree, fout)
        except UnknownNode:
            # Try next game tree in this file
            if DEBUG:
                print >>sys.stderr, "Unknown Node"
            continue

if __name__ == "__main__":
    if not len(args['FILENAMES']):
        process_sgf_file(sys.stdin, sys.stdout)
    else:
        for in_filename in args['FILENAMES']:
            if args['stdout_only']:
                fout = sys.stdout
            else:
                if re.search('sgf$', in_filename):
                    filename_base = in_filename[:-3]
                else:
                    filename_base = in_filename
                # Save the .gtp file
                out_filename = filename_base + 'gtp'

                fout = open(out_filename, 'w')

            fin = open(in_filename, 'r')

            process_sgf_file(fin, fout)

            fin.close()
            if not args['stdout_only']:
                fout.close()




#!/usr/local/bin/python

# sgflib.py (Smart Game Format Parser Library)
# Copyright (C) 2000  David John Goodger (dgoodger@bigfoot.com)
# 
# This library is free software; you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License as published by the
# Free Software Foundation; either version 2 of the License, or (at your
# option) any later version.
# 
# This library is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License
# for more details.
# 
# You should have received a copy of the GNU Lesser General Public License
# (lgpl.txt) along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# The license is currently available on the Internet at:
#     http://www.gnu.org/copyleft/lesser.html

"""
=============================================
 Smart Game Format Parser Library: sgflib.py
=============================================
version 1.0 (2000-03-27)

Homepage: [[http://gotools.sourceforge.net]]

Copyright (C) 2000 David John Goodger ([[mailto:dgoodger@bigfoot.com]]; davidg
on NNGS, IGS, goclub.org). sgflib.py comes with ABSOLUTELY NO WARRANTY. This is
free software, and you are welcome to redistribute it and/or modify it under the
terms of the GNU Lesser General Public License; see the source code for details.

Description
===========
This library contains a parser and classes for SGF, the Smart Game Format. SGF
is a text only, tree based file format designed to store game records of board
games for two players, most commonly for the game of go. (See the official SGF
specification at [[http://www.POBoxes.com/sgf/]]).

Given a string containing a complete SGF data instance, the 'SGFParser' class
will create a 'Collection' object consisting of one or more 'GameTree''s (one
'GameTree' per game in the SGF file), each containing a sequence of 'Node''s and
(potentially) two or more variation 'GameTree''s (branches). Each 'Node'
contains an ordered dictionary of 'Property' ID/value pairs (note that values
are lists, and can have multiple entries).

Tree traversal methods are provided through the 'Cursor' class.

The default representation (using 'str()' or 'print') of each class of SGF
objects is the Smart Game Format itself."""


# Revision History:
#
# 1.0 (2000-03-27): First public release.
# - Ready for prime time.
#
# 0.1 (2000-01-16):
# - Initial idea & started coding.


import string, re
from typelib import List, Dictionary


# Parsing Exceptions

class EndOfDataParseError(Exception):
	""" Raised by 'SGFParser.parseVariations()', 'SGFParser.parseNode()'."""
	pass

class GameTreeParseError(Exception):
	""" Raised by 'SGFParser.parseGameTree()'."""
	pass

class NodePropertyParseError(Exception):
	""" Raised by 'SGFParser.parseNode()'."""
	pass

class PropertyValueParseError(Exception):
	""" Raised by 'SGFParser.parsePropertyValue()'."""
	pass

# Tree Construction Exceptions

class DirectAccessError(Exception):
	""" Raised by 'Node.__setitem__()', 'Node.update()'."""
	pass
	
class DuplicatePropertyError(Exception):
	""" Raised by 'Node.addProperty()'."""
	pass

# Tree Navigation Exceptions
class GameTreeNavigationError(Exception):
	""" Raised by 'Cursor.next()'."""
	pass

class GameTreeEndError(Exception):
	""" Raised by 'Cursor.next()', 'Cursor.previous()'."""
	pass


# for type checking
INT_TYPE 			= type(0)					# constant

# miscellaneous constants
MAX_LINE_LEN		= 76						# constant; for line breaks


class SGFParser:
	"""
	Parser for SGF data. Creates a tree structure based on the SGF standard
	itself. 'SGFParser.parse()' will return a 'Collection' object for the entire
	data.

	Instance Attributes:
	- self.data : string -- The complete SGF data instance.
	- self.datalen : integer -- Length of 'self.data'.
	- self.index : integer -- Current parsing position in 'self.data'.

	Class Attributes:
	- re* : re.RegexObject -- Regular expression text matching patterns.
	- ctrltrans: string[256] -- Control character translation table for
	  string.translate(), used to remove all control characters from Property
	  values. May be overridden (preferably in instances)."""

	# text matching patterns
	reGameTreeStart 	= re.compile(r'\s*\(')
	reGameTreeEnd 		= re.compile(r'\s*\)')
	reGameTreeNext 		= re.compile(r'\s*(;|\(|\))')
	reNodeContents 		= re.compile(r'\s*([A-Za-z]+(?=\s*\[))')
	rePropertyStart 	= re.compile(r'\s*\[')
	rePropertyEnd 		= re.compile(r'\]')
	reEscape 			= re.compile(r'\\')
	reLineBreak 		= re.compile(r'\r\n?|\n\r?')	# CR, LF, CR/LF, LF/CR


	# character translation tables
	# for control characters (except LF \012 & CR \015): convert to spaces
	ctrltrans 			= string.maketrans("\000\001\002\003\004\005\006\007" +
						 "\010\011\013\014\016\017\020\021\022\023\024\025\026\027" +
						 "\030\031\032\033\034\035\036\037", " "*30)

	def __init__(self, data):
		""" Initialize the instance attributes. See the class itself for info."""
		self.data = data
		self.datalen = len(data)
		self.index = 0

	def parse(self):
		""" Parses the SGF data stored in 'self.data', and returns a 'Collection'."""
		c = Collection()
		while self.index < self.datalen:
			g = self.parseOneGame()
			if g:
				c.append(g)
			else:
				break
		return c

	def parseOneGame(self):
		""" Parses one game from 'self.data'. Returns a 'GameTree' containing
			one game, or 'None' if the end of 'self.data' has been reached."""
		if self.index < self.datalen:
			match = self.reGameTreeStart.match(self.data, self.index)
			if match:
				self.index = match.end()
				return self.parseGameTree()
		return None

	def parseGameTree(self):
		""" Called when "(" encountered, ends when a matching ")" encountered.
			Parses and returns one 'GameTree' from 'self.data'. Raises
			'GameTreeParseError' if a problem is encountered."""
		g = GameTree()
		while self.index < self.datalen:
			match = self.reGameTreeNext.match(self.data, self.index)
			if match:
				self.index = match.end()
				if match.group(1) == ";":				# found start of node
					if g.variations:
						raise GameTreeParseError(
									"A node was encountered after a variation.")
					g.append(g.makeNode(self.parseNode()))
				elif match.group(1) == "(":				# found start of variation
					g.variations = self.parseVariations()
				else:									# found end of GameTree ")"
					return g
			else:										# error
				raise GameTreeParseError
		return g

	def parseVariations(self):
		""" Called when "(" encountered inside a 'GameTree', ends when a
			non-matching ")" encountered. Returns a list of variation
			'GameTree''s. Raises 'EndOfDataParseError' if the end of 'self.data'
			is reached before the end of the enclosing 'GameTree'."""
		v = []
		while self.index < self.datalen:
			# check for ")" at end of GameTree, but don't consume it
			match = self.reGameTreeEnd.match(self.data, self.index)
			if match:
				return v
			g = self.parseGameTree()
			if g:
				v.append(g)
			# check for next variation, and consume "("
			match = self.reGameTreeStart.match(self.data, self.index)
			if match:
				self.index = match.end()
		raise EndOfDataParseError

	def parseNode(self):
		""" Called when ";" encountered (& is consumed). Parses and returns one
			'Node', which can be empty. Raises 'NodePropertyParseError' if no
			property values are extracted. Raises 'EndOfDataParseError' if the
			end of 'self.data' is reached before the end of the node (i.e., the
			start of the next node, the start of a variation, or the end of the
			enclosing game tree)."""
		n = Node()
		while self.index < self.datalen:
			match = self.reNodeContents.match(self.data, self.index)
			if match:
				self.index = match.end()
				pvlist = self.parsePropertyValue()
				if pvlist:
					n.addProperty(n.makeProperty(match.group(1), pvlist))
				else:
					raise NodePropertyParseError
			else:										# reached end of Node
				return n
		raise EndOfDataParseError

	def parsePropertyValue(self):
		""" Called when "[" encountered (but not consumed), ends when the next
			property, node, or variation encountered. Parses and returns a list
			of property values. Raises 'PropertyValueParseError' if there is a
			problem."""
		pvlist = []
		while self.index < self.datalen:
			match = self.rePropertyStart.match(self.data, self.index)
			if match:
				self.index = match.end()
				v = ""									# value
				# scan for escaped characters (using '\'), unescape them (remove linebreaks)
				mend = self.rePropertyEnd.search(self.data, self.index)
				mesc = self.reEscape.search(self.data, self.index)
				while mesc and mend and (mesc.end() < mend.end()):
														# copy up to '\', but remove '\'
					v = v + self.data[self.index:mesc.start()]
					mbreak = self.reLineBreak.match(self.data, mesc.end())
					if mbreak:
						self.index = mbreak.end()		# remove linebreak
					else:
						v = v + self.data[mesc.end()]	# copy escaped character
						self.index = mesc.end() + 1		# move to point after escaped char
					mend = self.rePropertyEnd.search(self.data, self.index)
					mesc = self.reEscape.search(self.data, self.index)
				if mend:
					v = v + self.data[self.index:mend.start()]
					self.index = mend.end()
					pvlist.append(self._convertControlChars(v))
				else:
					raise PropertyValueParseError
			else:										# reached end of Property
				break
		if len(pvlist) >= 1:
			return pvlist
		else:
			raise PropertyValueParseError

	def _convertControlChars(self, text):
		""" Converts control characters in 'text' to spaces, using the
			'self.ctrltrans' translation table. Override for variant
			behaviour."""
		return string.translate(text, self.ctrltrans)


class RootNodeSGFParser(SGFParser):
	""" For parsing only the first 'GameTree''s root Node of an SGF file."""

	def parseNode(self):
		""" Calls 'SGFParser.parseNode()', sets 'self.index' to point to the end
			of the data (effectively ending the 'GameTree' and 'Collection'),
			and returns the single (root) 'Node' parsed."""
		n = SGFParser.parseNode(self)			# process one Node as usual
		self.index = self.datalen				# set end of data
		return n								# we're only interested in the root node


class Collection(List):
	"""
	An SGF collection: multiple 'GameTree''s. Instance atributes:
	- self[.data] : list of 'GameTree' -- One 'GameTree' per game."""

	def __str__(self):
		""" SGF representation. Separates game trees with a blank line."""
		return string.join(map(str, self.data), "\n"*2)

	def cursor(self, gamenum=0):
		""" Returns a 'Cursor' object for navigation of the given 'GameTree'."""
		return Cursor(self[gamenum])


class GameTree(List):
	"""
	An SGF game tree: a game or variation. Instance attributes:
	- self[.data] : list of 'Node' -- game tree 'trunk'.
	- self.variations : list of 'GameTree' -- 0 or 2+ variations.
	  'self.variations[0]' contains the main branch (sequence actually played)."""

	def __init__(self, nodelist=None, variations=None):
		"""
			Initialize the 'GameTree'. Arguments:
			- nodelist : 'GameTree' or list of 'Node' -- Stored in 'self.data'.
			- variations : list of 'GameTree' -- Stored in 'self.variations'."""
		List.__init__(self, nodelist)
		self.variations = variations or []

	def __str__(self):
		""" SGF representation, with proper line breaks between nodes."""
		if len(self):
			s = "(" + str(self[0])				# append the first Node automatically
			l = len(string.split(s, "\n")[-1])	# accounts for line breaks within Nodes
			for n in map(str, self[1:]):
				if l + len(string.split(n, "\n")[0]) > MAX_LINE_LEN:
					s = s + "\n"
					l = 0
				s = s + n
				l = len(string.split(s, "\n")[-1])
			return s + string.join(map(str, [""] + self.variations), "\n") + ")"
		else:
			return ""							# empty GameTree illegal; "()" illegal

	def mainline(self):
		""" Returns the main line of the game (variation A) as a 'GameTree'."""
		if self.variations:
			return GameTree(self.data + self.variations[0].mainline())
		else:
			return self

	def makeNode(self, plist):
		"""
			Create a new 'Node' containing the properties contained in 'plist'.
			Override/extend to create 'Node' subclass instances (move, setup).
			Argument:
			- plist : 'Node' or list of 'Property'"""
		return Node(plist)

	def cursor(self):
		""" Returns a 'Cursor' object for navigation of this 'GameTree'."""
		return Cursor(self)

	def propertySearch(self, pid, getall=0):
		"""
			Searches this 'GameTree' for nodes containing matching properties.
			Returns a 'GameTree' containing the matched node(s). Arguments:
			- pid : string -- ID of properties to search for.
			- getall : boolean -- Set to true (1) to return all 'Node''s that
			  match, or to false (0) to return only the first match."""
		matches = []
		for n in self:
			if n.has_key(pid):
				matches.append(n)
				if not getall:
					break
		else:	# getall or not matches:
			for v in self.variations:
				matches = matches + v.propertySearch(pid, getall)
				if not getall and matches:
					break
		return GameTree(matches)


class Node(Dictionary):
	"""
	An SGF node. Instance Attributes:
	- self[.data] : ordered dictionary -- '{Property.id:Property}' mapping.
	  (Ordered dictionary: allows offset-indexed retrieval). Properties *must*
	  be added using 'self.addProperty()'.

	Example: Let 'n' be a 'Node' parsed from ';B[aa]BL[250]C[comment]':
	- 'str(n["BL"])'  =>  '"BL[250]"'
	- 'str(n[0])'     =>  '"B[aa]"'
	- 'map(str, n)'   =>  '["B[aa]","BL[250]","C[comment]"]'"""

	def __init__(self, plist=[]):
		"""
			Initializer. Argument:
			- plist: Node or list of 'Property'."""
		Dictionary.__init__(self)
		self.order = []
		for p in plist:
			self.addProperty(p)

	def __getitem__(self, key):
		""" On 'self[key]', 'x in self', 'for x in self'. Implements all
			indexing-related operations. Allows both key- and offset-indexed
			retrieval. Membership and iteration ('in', 'for') repeatedly index
			from 0 until 'IndexError'."""
		if type(key) is INT_TYPE:
			return self.order[key]
		else:
			return self.data[key]

	def __setitem__(self, key, x):
		""" On 'self[key]=x'. Allows assignment to existing items only. Raises 
			'DirectAccessError' on new item assignment."""
		if self.has_key(key):
			self.order[self.order.index(self[key])] = x
			Dictionary.__setitem__(self, key, x)
		else:
			raise DirectAccessError(
				"Properties may not be added directly; use addProperty() instead.")

	def __delitem__(self, key):
		""" On 'del self[key]'. Updates 'self.order' to maintain consistency."""
		self.order.remove(self[key])
		Dictionary.__delitem__(self, key)

	def __getslice__(self, low, high):
		""" On 'self[low:high]'."""
		return self.order[low:high]

	def __str__(self):
		""" SGF representation, with proper line breaks between properties."""
		if len(self):
			s = ";" + str(self[0])
			l = len(string.split(s, "\n")[-1])	# accounts for line breaks within Properties
			for p in map(str, self[1:]):
				if l + len(string.split(p, "\n")[0]) > MAX_LINE_LEN:
					s = s + "\n"
					l = 0
				s = s + p
				l = len(string.split(s, "\n")[-1])
			return s
		else:
			return ";"

	def update(self, dict):
		""" 'Dictionary' method not applicable to 'Node'. Raises 
			'DirectAccessError'."""
		raise DirectAccessError(
			"The update() method is not supported by Node; use addProperty() instead.")

	def addProperty(self, property):
		"""
			Adds a 'Property' to this 'Node'. Checks for duplicate properties
			(illegal), and maintains the property order. Argument:
			- property : 'Property'"""
		if self.has_key(property.id):
			raise DuplicatePropertyError
		else:
			self.data[property.id] = property
			self.order.append(property)

	def makeProperty(self, id, valuelist):
		"""
			Create a new 'Property'. Override/extend to create 'Property'
			subclass instances (move, setup, game-info, etc.). Arguments:
			- id : string
			- valuelist : 'Property' or list of values"""
		return Property(id, valuelist)


class Property(List):
	"""
	An SGF property: a set of label and value(s). Instance attributes:
	- self[.data] : list -- property values.
	- self.id : string -- SGF standard property label.
	- self.name : string -- actual label used in the SGF data. For example, the
	  property 'CoPyright[...]' has name 'CoPyright' and id 'CP'."""

	def __init__(self, id, values, name=None):
		"""
			Initialize the 'Property'. Arguments:
			- id : string
			- name : string (optional) -- If not given, 'self.name' 
			- nodelist : 'GameTree' or list of 'Node' -- Stored in 'self.data'.
			- variations : list of 'GameTree' -- Stored in 'self.variations'."""
		List.__init__(self, values)				# XXX will _convert work here?
		self.id = id
		self.name = name or id

	def __str__(self):
		return self.name + "[" + string.join(map(_escapeText, self), "][") + "]"


class Cursor:
	"""
	'GameTree' navigation tool. Instance attributes:
	- self.game : 'GameTree' -- The root 'GameTree'.
	- self.gametree : 'GameTree' -- The current 'GameTree'.
	- self.node : 'Node' -- The current Node.
	- self.nodenum : integer -- The offset of 'self.node' from the root of
	  'self.game'. The nodenum of the root node is 0.
	- self.index : integer -- The offset of 'self.node' within 'self.gametree'.
	- self.stack : list of 'GameTree' -- A record of 'GameTree''s traversed.
	- self.children : list of 'Node' -- All child nodes of the current node.
	- self.atEnd : boolean -- Flags if we are at the end of a branch.
	- self.atStart : boolean -- Flags if we are at the start of the game."""

	def __init__(self, gametree):
		""" Initialize root 'GameTree' and instance variables."""
		self.game = gametree					# root GameTree
		self.reset()

	def reset(self):
		""" Set 'Cursor' to point to the start of the root 'GameTree', 'self.game'."""
		self.gametree = self.game
		self.nodenum = 0
		self.index = 0
		self.stack = []
		self.node = self.gametree[self.index]
		self._setChildren()
		self._setFlags()

	def next(self, varnum=0):
		"""
			Moves the 'Cursor' to & returns the next 'Node'. Raises
			'GameTreeEndError' if the end of a branch is exceeded. Raises
			'GameTreeNavigationError' if a non-existent variation is accessed.
			Argument:
			- varnum : integer, default 0 -- Variation number. Non-zero only
			  valid at a branching, where variations exist."""
		if self.index + 1 < len(self.gametree):	# more main line?
			if varnum != 0:
				raise GameTreeNavigationError("Nonexistent variation.")
			self.index = self.index + 1
		elif self.gametree.variations:			# variations exist?
			if varnum < len(self.gametree.variations):
				self.stack.append(self.gametree)
				self.gametree = self.gametree.variations[varnum]
				self.index = 0
			else:
				raise GameTreeNavigationError("Nonexistent variation.")
		else:
			raise GameTreeEndError
		self.node = self.gametree[self.index]
		self.nodenum = self.nodenum + 1
		self._setChildren()
		self._setFlags()
		return self.node

	def previous(self):
		""" Moves the 'Cursor' to & returns the previous 'Node'. Raises
			'GameTreeEndError' if the start of a branch is exceeded."""
		if self.index - 1 >= 0:					# more main line?
			self.index = self.index - 1
		elif self.stack:						# were we in a variation?
			self.gametree = self.stack.pop()
			self.index = len(self.gametree) - 1
		else:
			raise GameTreeEndError
		self.node = self.gametree[self.index]
		self.nodenum = self.nodenum - 1
		self._setChildren()
		self._setFlags()
		return self.node

	def _setChildren(self):
		""" Sets up 'self.children'."""
		if self.index + 1 < len(self.gametree):
			self.children = [self.gametree[self.index+1]]
		else:
			self.children = map(lambda list: list[0], self.gametree.variations)

	def _setFlags(self):
		""" Sets up the flags 'self.atEnd' and 'self.atStart'."""
		self.atEnd = not self.gametree.variations and (self.index + 1 == len(self.gametree))
		self.atStart = not self.stack and (self.index == 0)


reCharsToEscape = re.compile(r'\]|\\')			# characters that need to be \escaped

def _escapeText(text):
	""" Adds backslash-escapes to property value characters that need them."""
	output = ""
	index = 0
	match = reCharsToEscape.search(text, index)
	while match:
		output = output + text[index:match.start()] + '\\' + text[match.start()]
		index = match.end()
		match = reCharsToEscape.search(text, index)
	output = output + text[index:]
	return output


def selfTest1(onConsole=0):
	""" Canned data test case"""
	sgfdata = r"""  	 (;GM [1]US[someone]CoPyright[\
  Permission to reproduce this game is given.]GN[a-b]EV[None]RE[B+Resign]
PW[a]WR[2k*]PB[b]BR[4k*]PC[somewhere]DT[2000-01-16]SZ[19]TM[300]KM[4.5]
HA[3]AB[pd][dp][dd];W[pp];B[nq];W[oq]C[ x started observation.
](;B[qc]C[ [b\]: \\ hi x! ;-) \\];W[kc])(;B[hc];W[oe]))   """
	print "\n\n********** Self-Test 1 **********\n"
	print "Input data:\n"
	print sgfdata
	print "\n\nParsed data: "
	col = SGFParser(sgfdata).parse()
	print "done\n"
	cstr = str(col)
	print cstr, "\n"
	print "Mainline:\n"
	m = col[0].mainline()
	print m, "\n"
	##print "as GameTree:\n"
	##print GameTree(m), "\n"
	print "Tree traversal (forward):\n"
	c = col.cursor()
	while 1:
		print "nodenum: %s; index: %s; children: %s; node: %s" % (c.nodenum, c.index, len(c.children), c.node)
		if c.atEnd: break
		c.next()
	print "\nTree traversal (backward):\n"
	while 1:
		print "nodenum: %s; index: %s; children: %s; node: %s" % (c.nodenum, c.index, len(c.children), c.node)
		if c.atStart: break
		c.previous()
	print "\nSearch for property 'B':"
	print col[0].propertySearch("B", 1)
	print "\nSearch for property 'C':"
	print col[0].propertySearch("C", 1)
	pass

def selfTest2(onConsole=0):
	""" Macintosh-based SGF file test"""
	import macfs
	print "\n\n********** Self-Test 2 (Mac) **********\n"
	thefile = macfs.PromptGetFile("Please choose an SGF file:")
	if not thefile[1]:
		return
	srcpath = thefile[0].as_pathname()
	src = open(srcpath, 'r')
	sgfdata = src.read()
	print "Input data:\n"
	print sgfdata
	print "\n\nParsed data:"
	col = SGFParser(sgfdata).parse()
	print "done\n"
	print str(col)


if __name__ == '__main__':
	print __doc__								# show module's documentation string
	selfTest1()
	import os
	if os.name == 'mac':
		selfTest2()



import numpy as np
cimport numpy as np
cimport cython
from libc.stdlib cimport malloc, free
from libc.stdio cimport FILE
from libcpp.vector cimport vector
from libcpp.string cimport string
from cython.operator cimport dereference as deref
from cpython.ref cimport PyObject


##### Wrapper API declarations #####
# These must go first before Pachi imports, because they use extern "C" on Pachi headers
cdef extern from "ptr.hpp":
    cppclass ptr[T]:
        ptr()
        T* get()
        void reset(T*)
        void assign(const ptr[T]&)

# Exceptions wrappers for C++ code
# see http://stackoverflow.com/a/29002414
class IllegalMove(RuntimeError):
    pass
class PachiEngineError(RuntimeError):
    pass
cdef public PyObject* _PyIllegalMove = <PyObject*> IllegalMove
cdef public PyObject* _PyPachiEngineError = <PyObject*> PachiEngineError
cdef extern from "exceptions.hpp":
    void raise_py_error()

# C++ wrapper for Pachi
cdef extern from "goutil.hpp":
    enum stone:
        S_NONE
        S_BLACK
        S_WHITE
        S_OFFBOARD
        S_MAX
    ctypedef int coord_t

    cppclass PachiBoard:
        board* pachiboard()
        ptr[PachiBoard] clone()
        int size()
    ctypedef ptr[PachiBoard] PachiBoardPtr
    PachiBoardPtr CreatePachiBoard(int size)

    void GetLegalMoves(PachiBoardPtr b, stone color, bint filter_suicides, vector[coord_t]* out)
    bint IsTerminal(PachiBoardPtr)
    float FastScore(PachiBoardPtr);
    float OfficialScore(PachiBoardPtr);
    string ToString(PachiBoardPtr)
    void PlayInPlace(PachiBoardPtr b, const move& m) except +raise_py_error
    PachiBoardPtr Play(PachiBoardPtr, const move&) except +raise_py_error
    PachiBoardPtr PlayRandom(PachiBoardPtr b, stone color, coord_t *coord)


    # Coordinate representation conversion
    # x/y are Pachi coordinates
    # i/j are the encoding used for training
    int i_from_xy(board* b, int x, int y)
    int j_from_xy(board* b, int x, int y)
    stone board_atij(board* b, int i, int j)
    int coord_ij(board* b, int i, int j)
    int i_from_coord(board* b, coord_t c)
    int j_from_coord(board* b, coord_t c)

    cppclass PachiEngine:
        PachiEngine(PachiBoardPtr b, const string& engine_type, string arg) except +raise_py_error
        coord_t genmove(PachiBoardPtr curr_board, stone curr_color, const string& timestr) except +raise_py_error
        void notify(PachiBoardPtr curr_board, coord_t move_coord, stone move_color)


##### Pachi API declarations #####

cdef extern from "stone.h":
    enum stone:
        S_NONE
        S_BLACK
        S_WHITE
        S_OFFBOARD
        S_MAX

    stone str2stone(char *str)
    stone pachi_stone_other "stone_other" (stone s)

cdef extern from "move.h":
    ctypedef int coord_t

    # void coord_done(coord_t *c)

    # # Return coordinate string in a dynamically allocated buffer. Thread-safe.
    char *coord2str(coord_t c, board *b)
    coord_t *str2coord(char *str, int board_size)

    struct move:
        coord_t coord
        stone color

    # Special moves
    coord_t pass_coord "pass"
    coord_t resign_coord "resign"

cdef extern from "board.h":
    int GROUP_REFILL_LIBS
    struct group:
        int libs
    ctypedef coord_t group_t

    struct board:
        pass
    int board_size(board*) # includes padding, so returns 19 + 2
    # stone board_at(board* b, int c)
    # stone board_atxy(board* b, int x, int y)
    # group_t group_at(board* b, int c)
    # group board_group_info(board* b, int g)

    void board_handicap(board *board, int stones, FILE *f)
    bint board_set_rules(board *board, char *name)

cdef extern from "random.h":
    void fast_srandom(unsigned long seed)


# Internal utility functions


cdef enum Channel:
    CHAN_CELL_BLACK = 0
    CHAN_CELL_WHITE
    CHAN_CELL_EMPTY
cdef int _NUM_FEATURE_CHANNELS = 3
cdef void encode_board(board* b, np.ndarray[np.int_t, ndim=3] out_x):
    cdef int s = board_size(b) - 2
    assert out_x.shape[0] == _NUM_FEATURE_CHANNELS and out_x.shape[1] == out_x.shape[2] == s
    out_x[...] = 0

    cdef unsigned int i, j
    cdef stone curr_stone
    for i in range(s):
        for j in range(s):
            curr_stone = board_atij(b, i, j)
            # Cell color
            if curr_stone == S_BLACK:
                out_x[<unsigned int>CHAN_CELL_BLACK,i,j] = 1
            elif curr_stone == S_WHITE:
                out_x[<unsigned int>CHAN_CELL_WHITE,i,j] = 1
            elif curr_stone == S_NONE:
                out_x[<unsigned int>CHAN_CELL_EMPTY,i,j] = 1
            else:
                assert False

# Python board wrapper
cdef PyPachiBoard wrap_board(PachiBoardPtr b):
    return PyPachiBoard()._set(b)

cdef class PyPachiBoard:
    cdef PachiBoardPtr _bptr
    cdef PachiBoard* _b # for convenience. set to self._bptr.get()
    cdef int _size

    cdef _set(self, PachiBoardPtr b):
        self._bptr.assign(b)
        self._b = self._bptr.get()
        self._size = self._b.size()
        return self

    def __repr__(self):
        return ToString(self._bptr)

    def clone(self):
        return wrap_board(self._bptr.get().clone())

    def get_stones(self, stone color):
        cdef np.ndarray[np.int_t, ndim=2] stones = np.empty((self._size*self._size, 2), dtype=np.int)
        cdef int i, j
        cdef unsigned int num_stones = 0
        for i in range(self._size):
            for j in range(self._size):
                if board_atij(self._b.pachiboard(), i, j) == color:
                    stones[num_stones,0] = i
                    stones[num_stones,1] = j
                    num_stones += 1
        return stones[:num_stones,:]

    def get_legal_coords(self, stone color, bint filter_suicides=False):
        cdef vector[coord_t] legal_coords
        GetLegalMoves(self._bptr, color, filter_suicides, &legal_coords)
        return legal_coords

    def encode(self, np.ndarray[np.int_t, ndim=3] out_x=None):
        if out_x is None:
            out_x = np.zeros((NUM_FEATURE_CHANNELS, self._size, self._size), dtype=int)
        encode_board(self._b.pachiboard(), out_x)
        return out_x

    def play(self, int coord, stone color):
        assert color == S_BLACK or color == S_WHITE
        cdef move m
        m.color = color
        m.coord = coord
        return wrap_board(Play(self._bptr, m))

    def play_random(self, stone color, bint return_action=False):
        cdef coord_t c
        cdef PyPachiBoard out = wrap_board(PlayRandom(self._bptr, color, &c))
        if not return_action: return out
        return out, c

    def play_inplace(self, int coord, stone color):
        assert color == S_BLACK or color == S_WHITE
        cdef move m
        m.color = color
        m.coord = coord
        PlayInPlace(self._bptr, m)

    property size:
        def __get__(self):
            return self._size
    property num_stones:
        def __get__(self):
            cdef int i, j, n = 0
            for i in range(self._size):
                for j in range(self._size):
                    if board_atij(self._b.pachiboard(), i, j) != S_NONE:
                        n += 1
            return n
    property empty:
        def __get__(self):
            return self.num_stones == 0
    property black_stones:
        def __get__(self):
            return self.get_stones(S_BLACK)
    property white_stones:
        def __get__(self):
            return self.get_stones(S_WHITE)
    property is_terminal:
        def __get__(self):
            return IsTerminal(self._bptr)
    property fast_score:
        def __get__(self):
            return FastScore(self._bptr)
    property official_score:
        def __get__(self):
            return OfficialScore(self._bptr)

    def __richcmp__(PyPachiBoard self, PyPachiBoard other, int op):
        # TODO: use the C++ operator==
        if op != 2 and op != 3: return NotImplemented
        if other._size != self._size: return False
        cdef int i, j
        for i in range(self._size):
            for j in range(self._size):
                if board_atij(self._b.pachiboard(), i, j) != board_atij(other._b.pachiboard(), i, j):
                    return op == 3
        return op == 2

    def __hash__(self):
        # Not sure if this is the best hash implementation, but it seems to work
        cdef long h = 0
        cdef int i, j
        for i in range(self._size):
            for j in range(self._size):
                h = 101*h + board_atij(self._b.pachiboard(), i, j)
        return h

    def __getitem__(self, idx):
        if len(idx) != 2:
            raise IndexError('Must provide 2 indices')
        cdef int i = idx[0]
        cdef int j = idx[1]
        if not (0 <= i < self._size and 0 <= j < self._size):
            raise IndexError('Coordinates %d,%d out of range for board size %d' % (i, j, self._size))
        return board_atij(self._b.pachiboard(), i, j)

    ### Utilities ###

    def str_to_coord(self, char *s):
        cdef coord_t *cptr = str2coord(s, board_size(self._b.pachiboard()))
        cdef coord_t c = deref(cptr)
        free(cptr)
        # Sanity checking
        if c == pass_coord or c == resign_coord: return c
        if not (0 <= i_from_coord(self._b.pachiboard(), c) < self._size and
                0 <= j_from_coord(self._b.pachiboard(), c) < self._size):
            raise RuntimeError('Invalid coordinate %s' % s)
        return c

    def coord_to_str(self, coord_t c):
        cdef char* tmpstr = coord2str(c, self._b.pachiboard())
        cdef string s
        s += tmpstr
        free(tmpstr)
        return s

    def coord_to_ij(self, coord_t c):
        cdef int i = i_from_coord(self._b.pachiboard(), c)
        cdef int j = j_from_coord(self._b.pachiboard(), c)
        return i, j

    def ij_to_coord(self, int i, int j):
        return coord_ij(self._b.pachiboard(), i, j)


cdef class PyPachiEngine:
    cdef PachiEngine* _engine

    def __cinit__(self, PyPachiBoard b, const string& engine_type, const string& arg):
        self._engine = new PachiEngine(b._bptr, engine_type, arg)

    def __dealloc__(self):
        del self._engine

    def genmove(self, PyPachiBoard curr_board, stone curr_color, const string& timestr):
        return self._engine.genmove(curr_board._bptr, curr_color, timestr)

    def notify(self, PyPachiBoard curr_board, coord_t move_coord, stone move_color):
        self._engine.notify(curr_board._bptr, move_coord, move_color)


##### Exposed constants #####
NUM_FEATURE_CHANNELS = _NUM_FEATURE_CHANNELS
WHITE = S_WHITE
BLACK = S_BLACK
EMPTY = S_NONE
PASS_COORD = pass_coord
RESIGN_COORD = resign_coord


##### Exposed functions #####
cpdef PyPachiBoard CreateBoard(int size):
    return wrap_board(CreatePachiBoard(size))

def pachi_srand(unsigned long seed):
    fast_srandom(seed)

def stone_other(stone s):
    return pachi_stone_other(s)

def color_to_str(stone s):
    if s == S_BLACK: return "black"
    elif s == S_WHITE: return "white"
    return "INVALID"



#! /usr/bin/env python

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
# This program is distributed with GNU Go, a Go program.            #
#                                                                   #
# Write gnugo@gnu.org or see http://www.gnu.org/software/gnugo/     #
# for more information.                                             #
#                                                                   #
# Copyright 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006 and 2007 #
# by the Free Software Foundation.                                  #
#                                                                   #
# This program is free software; you can redistribute it and/or     #
# modify it under the terms of the GNU General Public License       #
# as published by the Free Software Foundation - version 3,         #
# or (at your option) any later version.                            #
#                                                                   #
# This program is distributed in the hope that it will be           #
# useful, but WITHOUT ANY WARRANTY; without even the implied        #
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR           #
# PURPOSE.  See the GNU General Public License in file COPYING      #
# for more details.                                                 #
#                                                                   #
# You should have received a copy of the GNU General Public         #
# License along with this program; if not, write to the Free        #
# Software Foundation, Inc., 51 Franklin Street, Fifth Floor,       #
# Boston, MA 02111, USA.                                            #
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

from getopt import *
import popen2
import sys
import string
import re


debug = 0


def coords_to_sgf(size, board_coords):
    global debug
    
    board_coords = string.lower(board_coords)
    if board_coords == "pass":
        return ""
    if debug:
        print "Coords: <" + board_coords + ">"
    letter = board_coords[0]
    digits = board_coords[1:]
    if letter > "i":
        sgffirst = chr(ord(letter) - 1)
    else:
        sgffirst = letter
    sgfsecond = chr(ord("a") + int(size) - int(digits))
    return sgffirst + sgfsecond



class GTP_connection:

    #
    # Class members:
    #   outfile         File to write to
    #   infile          File to read from

    def __init__(self, command):
        try:
            infile, outfile = popen2.popen2(command)
        except:
            print "popen2 failed"
            sys.exit(1)
        self.infile  = infile
        self.outfile = outfile
        
    def exec_cmd(self, cmd):
        global debug
        
        if debug:
            sys.stderr.write("GTP command: " + cmd + "\n")
        self.outfile.write(cmd + "\n\n")
        self.outfile.flush()
        result = ""
        line = self.infile.readline()
        while line != "\n":
            result = result + line
            line = self.infile.readline()
        if debug:
            sys.stderr.write("Reply: " + line + "\n")

        # Remove trailing newline from the result
        if result[-1] == "\n":
            result = result[:-1]

        if len(result) == 0:
            return "ERROR: len = 0"
        if (result[0] == "?"):
            return "ERROR: GTP Command failed: " + result[2:]
        if (result[0] == "="):
            return result[2:]
        return "ERROR: Unrecognized answer: " + result
        

class GTP_player:

    # Class members:
    #    connection     GTP_connection

    def __init__(self, command):
        self.connection = GTP_connection(command)
        protocol_version = self.connection.exec_cmd("protocol_version")
        if protocol_version[:5] != "ERROR":
            self.protocol_version = protocol_version
        else:
            self.protocol_version = "1"

    def is_known_command(self, command):
        return self.connection.exec_cmd("known_command " + command) == "true"

    def genmove(self, color):
        if color[0] in ["b", "B"]:
            command = "black"
        elif color[0] in ["w", "W"]:
            command = "white"
        if self.protocol_version == "1":
            command = "genmove_" + command
        else:
            command = "genmove " + command

        return self.connection.exec_cmd(command)

    def black(self, move):
        if self.protocol_version == "1":
            self.connection.exec_cmd("black " + move)
        else:
            self.connection.exec_cmd("play black " + move)

    def white(self, move):
        if self.protocol_version == "1":
            self.connection.exec_cmd("white " + move)
        else:
            self.connection.exec_cmd("play white " + move)

    def komi(self, komi):
        self.connection.exec_cmd("komi " + komi)

    def boardsize(self, size):
        self.connection.exec_cmd("boardsize " + size)
        if self.protocol_version != "1":
            self.connection.exec_cmd("clear_board")

    def handicap(self, handicap, handicap_type):
        if handicap_type == "fixed":
            result = self.connection.exec_cmd("fixed_handicap %d" % (handicap))
        else:
            result = self.connection.exec_cmd("place_free_handicap %d"
                                              % (handicap))

        return string.split(result, " ")

    def loadsgf(self, endgamefile, move_number):
        self.connection.exec_cmd(string.join(["loadsgf", endgamefile,
                                 str(move_number)]))

    def list_stones(self, color):
        return string.split(self.connection.exec_cmd("list_stones " + color), " ")

    def quit(self):
        return self.connection.exec_cmd("quit")
    
    def showboard(self):
        board = self.connection.exec_cmd("showboard")
        if board and (board[0] == "\n"):
            board = board[1:]
        return board

    def get_random_seed(self):
        result = self.connection.exec_cmd("get_random_seed")
        if result[:5] == "ERROR":
            return "unknown"
        return result

    def set_random_seed(self, seed):
        self.connection.exec_cmd("set_random_seed " + seed)

    def get_program_name(self):
        return self.connection.exec_cmd("name") + " " + \
               self.connection.exec_cmd("version")

    def final_score(self):
        return self.connection.exec_cmd("final_score")

    def score(self):
        return self.final_score(self)

    def cputime(self):
        if (self.is_known_command("cputime")):
            return self.connection.exec_cmd("cputime")
        else:
            return "0"


class GTP_game:

    # Class members:
    #    whiteplayer     GTP_player
    #    blackplayer     GTP_player
    #    size            int
    #    komi            float
    #    handicap        int
    #    handicap_type   string
    #    handicap_stones int
    #    moves           list of string
    #    resultw
    #    resultb

    def __init__(self, whitecommand, blackcommand, size, komi, handicap,
                 handicap_type, endgamefile):
        self.whiteplayer = GTP_player(whitecommand)
        self.blackplayer = GTP_player(blackcommand)
        self.size = size
        self.komi = komi
        self.handicap = handicap
        self.handicap_type = handicap_type
        self.endgamefile = endgamefile
        self.sgffilestart = ""
        if endgamefile != "":
            self.init_endgame_contest_game()
        else:
            self.sgffilestart = ""

    def init_endgame_contest_game(self):
        infile = open(self.endgamefile)
        if not infile:
            print "Couldn't read " + self.endgamefile
            sys.exit(2)
        sgflines = infile.readlines()
        infile.close
        size = re.compile("SZ\[[0-9]+\]")
        move = re.compile(";[BW]\[[a-z]{0,2}\]")
        sgf_start = []
        for line in sgflines:
            match = size.search(line)
            if match:
                self.size = match.group()[3:-1]
            match = move.search(line)
            while match:
                sgf_start.append("A" + match.group()[1:])
                line = line[match.end():]
                match = move.search(line)
        self.endgame_start = len(sgf_start) - endgame_start_at
        self.sgffilestart = ";" + string.join(
                                    sgf_start[:self.endgame_start-1], "") + "\n"
        if self.endgame_start % 2 == 0:
            self.first_to_play = "W"
        else:
            self.first_to_play = "B"

    def get_position_from_engine(self, engine):
        black_stones = engine.list_stones("black")
        white_stones = engine.list_stones("white")
        self.sgffilestart = ";"
        if len(black_stones) > 0:
            self.sgffilestart += "AB"
            for stone in black_stones:
                self.sgffilestart += "[%s]" % coords_to_sgf(self.size, stone)
            self.sgffilestart += "\n"
        if len(white_stones) > 0:
            self.sgffilestart += "AW"
            for stone in white_stones:
                self.sgffilestart += "[%s]" % coords_to_sgf(self.size, stone)
            self.sgffilestart += "\n"

    def writesgf(self, sgffilename):
        "Write the game to an SGF file after a game"

        size = self.size
        outfile = open(sgffilename, "w")
        if not outfile:
            print "Couldn't create " + sgffilename
            return
        black_name = self.blackplayer.get_program_name()
        white_name = self.whiteplayer.get_program_name()
        black_seed = self.blackplayer.get_random_seed()
        white_seed = self.whiteplayer.get_random_seed()
        handicap = self.handicap
        komi     = self.komi
        result   = self.resultw

        outfile.write("(;GM[1]FF[4]RU[Japanese]SZ[%s]HA[%s]KM[%s]RE[%s]\n" %
                      (size, handicap, komi, result))
        outfile.write("PW[%s (random seed %s)]PB[%s (random seed %s)]\n" %
                      (white_name, white_seed, black_name, black_seed))
        outfile.write(self.sgffilestart)

        if handicap > 1:
            outfile.write("AB");
            for stone in self.handicap_stones:
                outfile.write("[%s]" %(coords_to_sgf(size, stone)))
            outfile.write("PL[W]\n")

        to_play = self.first_to_play

        for move in self.moves:
            sgfmove = coords_to_sgf(size, move)
            outfile.write(";%s[%s]\n" % (to_play, sgfmove))
            if to_play == "B":
                to_play = "W"
            else:
                to_play = "B"
        outfile.write(")\n")
        outfile.close

    def set_handicap(self, handicap):
        self.handicap = handicap

    def swap_players(self):
        temp = self.whiteplayer
        self.whiteplayer = self.blackplayer
        self.blackplayer = temp

    def play(self, sgffile):
        "Play a game"
        global verbose

        if verbose >= 1:
            print "Setting boardsize and komi for black\n"
        self.blackplayer.boardsize(self.size)
        self.blackplayer.komi(self.komi)

        if verbose >= 1:
            print "Setting boardsize and komi for white\n"
        self.whiteplayer.boardsize(self.size)
        self.whiteplayer.komi(self.komi)

        self.handicap_stones = []

        if self.endgamefile == "":
            if self.handicap < 2:
                self.first_to_play = "B"
            else:
                self.handicap_stones = self.blackplayer.handicap(self.handicap, self.handicap_type)
                for stone in self.handicap_stones:
                    self.whiteplayer.black(stone)
                self.first_to_play = "W"
        else:
            self.blackplayer.loadsgf(self.endgamefile, self.endgame_start)
            self.blackplayer.set_random_seed("0")
            self.whiteplayer.loadsgf(self.endgamefile, self.endgame_start)
            self.whiteplayer.set_random_seed("0")
            if self.blackplayer.is_known_command("list_stones"):
                self.get_position_from_engine(self.blackplayer)
            elif self.whiteplayer.is_known_command("list_stones"):
                self.get_position_from_engine(self.whiteplayer)

        to_play = self.first_to_play

        self.moves = []
        passes = 0
        won_by_resignation = ""
        while passes < 2:
            if to_play == "B":
                move = self.blackplayer.genmove("black")

                if move[:5] == "ERROR":
                    # FIXME: write_sgf
                    sys.exit(1)

                if move[:6] == "resign":
                    if verbose >= 1:
                        print "Black resigns"
                    won_by_resignation = "W+Resign"
                    break
                else:
                    self.moves.append(move)
                    if string.lower(move[:4]) == "pass":
                        passes = passes + 1
                        if verbose >= 1:
                            print "Black passes"
                    else:
                        passes = 0
                        self.whiteplayer.black(move)
                        if verbose >= 1:
                            print "Black plays " + move
                to_play = "W"
            else:
                move = self.whiteplayer.genmove("white")
                if move[:5] == "ERROR":
                    # FIXME: write_sgf
                    sys.exit(1)

                if move[:6] == "resign":
                    if verbose >= 1:
                        print "White resigns"
                    won_by_resignation = "B+Resign"
                    break
                else:
                    self.moves.append(move)
                    if string.lower(move[:4]) == "pass":
                        passes = passes + 1
                        if verbose >= 1:
                            print "White passes"
                    else:
                        passes = 0
                        self.blackplayer.white(move)
                        if verbose >= 1:
                            print "White plays " + move
                to_play = "B"

            if verbose >= 2:
                print self.whiteplayer.showboard() + "\n"

        if won_by_resignation == "":
            self.resultw = self.whiteplayer.final_score()
            self.resultb = self.blackplayer.final_score()
        else:
            self.resultw = won_by_resignation;
            self.resultb = won_by_resignation;
    #    if self.resultb == self.resultw:
    #        print "Result: ", self.resultw
    #    else:
    #        print "Result according to W: ", self.resultw
    #        print "Result according to B: ", self.resultb
        # FIXME:   $self->writesgf($sgffile) if defined $sgffile;
        if sgffile != "":
            self.writesgf(sgffile)

    def result(self):
        return (self.resultw, self.resultb)

    def cputime(self):
        cputime = {}
        cputime["white"] = self.whiteplayer.cputime()
        cputime["black"] = self.blackplayer.cputime()
        return cputime
    
    def quit(self):
        self.blackplayer.quit()
        self.whiteplayer.quit()


class GTP_match:

    # Class members:
    #    black
    #    white
    #    size
    #    komi
    #    handicap
    #    handicap_type

    def __init__(self, whitecommand, blackcommand, size, komi, handicap,
                 handicap_type, streak_length, endgamefilelist):
        self.white = whitecommand
        self.black = blackcommand
        self.size = size
        self.komi = komi
        self.handicap = handicap
        self.handicap_type = handicap_type
        self.streak_length = streak_length
        self.endgamefilelist = endgamefilelist

    def endgame_contest(self, sgfbase):
        results = []
        i = 1
        for endgamefile in self.endgamefilelist:
            game1 = GTP_game(self.white, self.black, self.size, self.komi,
                             0, "", endgamefile)
            game2 = GTP_game(self.black, self.white, self.size, self.komi,
                             0, "", endgamefile)
            if verbose:
                print "Replaying", endgamefile
                print "Black:", self.black
                print "White:", self.white
            game1.play("")
            result1 = game1.result()[0]
            if result1 != "0":
                plain_result1 = re.search(r"([BW]\+)([0-9]*\.[0-9]*)", result1)
                result1_float = float(plain_result1.group(2))
            else:
                plain_result1 = re.search(r"(0)", "0")
                result1_float = 0.0
            if result1[0] == "B":
                result1_float *= -1
            if verbose:
                print "Result:", result1
                print "Replaying", endgamefile
                print "Black:", self.white
                print "White:", self.black
            game2.play("")
            result2 = game2.result()[1]
            if verbose:
                print "Result:", result2
            if result2 != "0":
                plain_result2 = re.search(r"([BW]\+)([0-9]*\.[0-9]*)", result2)
                result2_float = float(plain_result2.group(2))
            else:
                plain_result2 = re.search(r"(0)", "0")
                result2_float = 0.0

            if result2[0] == "B":
                result2_float *= -1
            results.append(result1_float - result2_float)
            if (result1 != result2):
                print endgamefile+ ":", plain_result1.group(), \
                    plain_result2.group(), "Difference:",
                print result1_float - result2_float
            else:
                print endgamefile+": Same result:", plain_result1.group()
            sgffilename = "%s%03d" % (sgfbase, i)
            game1.writesgf(sgffilename + "_1.sgf")
            game2.writesgf(sgffilename + "_2.sgf")
            game1.quit()
            game2.quit()
            i += 1
        return results

    def play(self, games, sgfbase):
        last_color = ""
        last_streak = 0
        game = GTP_game(self.white, self.black,
                        self.size, self.komi, self.handicap,
                        self.handicap_type, "")
        results = []
        for i in range(games):
            sgffilename = "%s%03d.sgf" % (sgfbase, i + 1)
            game.play(sgffilename)
            result = game.result()
            if result[0] == result[1]:
                print "Game %d: %s" % (i + 1, result[0])
            else:
                print "Game %d: %s %s" % (i + 1, result[0], result[1])

            if result[0][0] == last_color:
                last_streak += 1
            elif result[0][0] != "0":
                last_color = result[0][0]
                last_streak = 1

            if last_streak == self.streak_length:
                if last_color == "W":
                    self.handicap += 1
                    if self.handicap == 1:
                        self.handicap = 2
                    print "White wins too often. Increasing handicap to %d" \
                          % (self.handicap)
                else:   
                    if self.handicap > 0:
                        self.handicap -= 1
                        if self.handicap == 1:
                            self.handicap = 0
                        print "Black wins too often. Decreasing handicap to %d" \
                              % (self.handicap)
                    else:
                        self.handicap = 2
                        game.swap_players()
                        print "Black looks stronger than white. Swapping colors and setting handicap to 2"
                game.set_handicap(self.handicap)
                last_color = ""
                last_streak = 0
            results.append(result)
        cputime = game.cputime()
        game.quit()
        return results, cputime


# ================================================================
#                      Main program
#


# Default values
#

white    = ""
black    = ""
komi     = ""
size     = "19"
handicap = 0
handicap_type = "fixed"
streak_length = -1
endgame_start_at = 0

games   = 1
sgfbase = "twogtp"

verbose = 0

helpstring = """

Run with:

twogtp --white \'<path to program 1> --mode gtp [program options]\' \\
       --black \'<path to program 2> --mode gtp [program options]\' \\
       [twogtp options]

Possible twogtp options:

  --verbose 1 (to list moves) or --verbose 2 (to draw board)
  --komi <amount>
  --handicap <amount>
  --free-handicap <amount>
  --adjust-handicap <length>        (change handicap by 1 after <length> wins
                                     in a row)
  --size <board size>               (default 19)
  --games <number of games to play> (default 1)
  --sgfbase <filename>              (create sgf files with sgfbase as basename)
  --endgame <moves before end>      (endgame contest - add filenames of
                                     games to be replayed after last option)
"""

def usage():
    print helpstring
    sys.exit(1)

try:
    (opts, params) = getopt(sys.argv[1:], "",
                            ["black=",
                             "white=",
                             "verbose=",
                             "komi=",
                             "boardsize=",
                             "size=",
                             "handicap=",
                             "free-handicap=",
                             "adjust-handicap=",
                             "games=",
                             "sgfbase=",
                             "endgame=",
                            ])
except:
    usage();

for opt, value in opts:
    if opt == "--black":
        black = value
    elif opt == "--white":
        white = value
    elif opt == "--verbose":
        verbose = int(value)
    elif opt == "--komi":
        komi = value
    elif opt == "--boardsize" or opt == "--size":
        size = value
    elif opt == "--handicap":
        handicap = int(value)
        handicap_type = "fixed"
    elif opt == "--free-handicap":
        handicap = int(value)
        handicap_type = "free"
    elif opt == "--adjust-handicap":
        streak_length = int(value)
    elif opt == "--games":
        games = int(value)
    elif opt == "--sgfbase":
        sgfbase = value
    elif opt == "--endgame":
        endgame_start_at = int(value)

if endgame_start_at != 0:
    endgame_filelist = params
else:
    endgame_filelist = []
    if params != []:
        usage()
            

if black == "" or white == "":
    usage()

if komi == "":
    if handicap > 1 and streak_length == -1:
        komi = "0.5"
    else:
        komi = "5.5"

match = GTP_match(white, black, size, komi, handicap, handicap_type,
                  streak_length, endgame_filelist)
if endgame_filelist != []:
    results = match.endgame_contest(sgfbase)
    win_black = 0
    win_white = 0
    for res in results:
        print res
        if res > 0.0:
            win_white += 1
        elif res < 0.0:
            win_black += 1
    print "%d games, %d wins for black, %d wins for white." \
         % (len(results), win_black, win_white)

else:
    results, cputimes = match.play(games, sgfbase)

    i = 0
    for resw, resb in results:
        i = i + 1
        if resw == resb:
            print "Game %d: %s" % (i, resw)
        else:
            print "Game %d: %s %s" % (i, resb, resw)
    if (cputimes["white"] != "0"):
        print "White: %ss CPU time" % cputimes["white"]
    if (cputimes["black"] != "0"):
        print "Black: %ss CPU time" % cputimes["black"]



#!/usr/local/bin/python

# typelib.py (Type Class Library)
# Copyright (c) 2000  David John Goodger
# 
# This software is provided "as-is", without any express or implied warranty.
# In no event will the authors be held liable for any damages arising from the
# use of this software.
# 
# Permission is granted to anyone to use this software for any purpose,
# including commercial applications, and to alter it and redistribute it
# freely, subject to the following restrictions:
# 
# 1. The origin of this software must not be misrepresented; you must not
# claim that you wrote the original software. If you use this software in a
# product, an acknowledgment in the product documentation would be appreciated
# but is not required.
# 
# 2. Altered source versions must be plainly marked as such, and must not be
# misrepresented as being the original software.
# 
# 3. This notice may not be removed or altered from any source distribution.

"""
================================
 Type Class Library: typelib.py
================================
version 1.0 (2000-03-27)

Homepage: [[http://gotools.sourceforge.net/]] (see sgflib.py)

Copyright (C) 2000 David John Goodger ([[mailto:dgoodger@bigfoot.com]]).
typelib.py comes with ABSOLUTELY NO WARRANTY. This is free software, and you are
welcome to redistribute it and/or modify it under certain conditions; see the
source code for details.

Description
===========
This library implements abstract superclasses to emulate Python's built-in data
types. This is useful when you want a class which acts like a built-in type, but
with added/modified behaviour (methods) and/or data (attributes).

Implemented types are: 'String', 'Tuple', 'List', 'Dictionary', 'Integer',
'Long', 'Float', 'Complex' (along with their abstract superclasses).

All methods, including special overloading methods, are implemented for each
type-emulation class. Instance data is stored internally in the 'data' attribute
(i.e., 'self.data'). The type the class is emulating is stored in the class
attribute 'self.TYPE' (as given by the built-in 'type(class)').  The
'SuperClass.__init__()' method uses two class-specific methods to instantiate
objects: '_reset()' and '_convert()'.

See "sgflib.py" (at module's homepage, see above) for examples of use. The Node
class is of particular interest: a modified 'Dictionary' which is ordered and
allows for offset-indexed retrieval."""


# Revision History
#
# 1.0 (2000-03-27): First public release.
# - Implemented Integer, Long, Float, and Complex.
# - Cleaned up a few loose ends.
# - Completed docstring documentatation.
#
# 0.1 (2000-01-27):
# - Implemented String, Tuple, List, and Dictionary emulation.
#
# To do:
# - Implement Function? File? (Have to come up with a good reason first ;-)


class SuperType:
	""" Superclass of all type classes. Implements methods common to all types.
		Concrete (as opposed to abstract) subclasses must define a class
		attribute 'self.TYPE' ('=type(Class)'), and methods '_reset(self)' and
		'_convert(self, data)'."""

	def __init__(self, data=None):
		"""
		On 'Class()', initialize 'self.data'. Argument:
		- 'data' : optional, default 'None' --
			- If the type of 'data' is identical to the Class' 'TYPE',
			  'data' will be shared (relevant for mutable types only).
			- If 'data' is given (and not false), it will be converted by
			  the Class-specific method 'self._convert(data)'. Incompatible
			  data types will raise an exception.
			- If 'data' is 'None', false, or not given, a Class-specific method
			  'self._reset()' is called to initialize an empty instance."""
		if data:
			if type(data) is self.TYPE:
				self.data = data
			else:
				self.data = self._convert(data)
		else:
			self._reset()

	def __str__(self):
		""" On 'str(self)' and 'print self'. Returns string representation."""
		return str(self.data)

	def __cmp__(self, x):
		"""	On 'self>x', 'self==x', 'cmp(self,x)', etc. Catches all
			comparisons: returns -1, 0, or 1 for less, equal, or greater."""
		return cmp(self.data, x)

	def __rcmp__(self, x):
		"""	On 'x>self', 'x==self', 'cmp(x,self)', etc. Catches all
			comparisons: returns -1, 0, or 1 for less, equal, or greater."""
		return cmp(x, self.data)

	def __hash__(self):
		""" On 'dictionary[self]', 'hash(self)'. Returns a unique and unchanging
			integer hash-key."""
		return hash(self.data)


class AddMulMixin:
	""" Addition & multiplication for numbers, concatenation & repetition for
		sequences."""

	def __add__(self, other):
		""" On 'self+other'. Numeric addition, or sequence concatenation."""
		return self.data + other

	def __radd__(self, other):
		""" On 'other+self'. Numeric addition, or sequence concatenation."""
		return other + self.data

	def __mul__(self, other):
		""" On 'self*other'. Numeric multiplication, or sequence repetition."""
		return self.data * other

	def __rmul__(self, other):
		""" On 'other*self'. Numeric multiplication, or sequence repetition."""
		return other * self.data


class MutableMixin:
	""" Assignment to and deletion of collection component."""

	def __setitem__(self, key, x):
		""" On 'self[key]=x'."""
		self.data[key] = x

	def __delitem__(self, key):
		""" On 'del self[key]'."""
		del self.data[key]


class ModMixin:
	""" Modulo remainder and string formatting."""

	def __mod__(self, other):
		""" On 'self%other'."""
		return self.data % other

	def __rmod__(self, other):
		""" On 'other%self'."""
		return other % self.data


class Number(SuperType, AddMulMixin, ModMixin):
	""" Superclass for numeric emulation types."""

	def __sub__(self, other):
		""" On 'self-other'."""
		return self.data - other

	def __rsub__(self, other):
		""" On 'other-self'."""
		return other - self.data

	def __div__(self, other):
		""" On 'self/other'."""
		return self.data / other

	def __rdiv__(self, other):
		""" On 'other/self'."""
		return other / self.data

	def __divmod__(self, other):
		""" On 'divmod(self,other)'."""
		return divmod(self.data, other)

	def __rdivmod__(self, other):
		""" On 'divmod(other,self)'."""
		return divmod(other, self.data)

	def __pow__(self, other, mod=None):
		""" On 'pow(self,other[,mod])', 'self**other'."""
		if mod is None:
			return self.data ** other
		else:
			return pow(self.data, other, mod)

	def __rpow__(self, other):
		""" On 'pow(other,self)', 'other**self'."""
		return other ** self.data

	def __neg__(self):
		""" On '-self'."""
		return -self.data

	def __pos__(self):
		""" On '+self'."""
		return +self.data

	def __abs__(self):
		""" On 'abs(self)'."""
		return abs(self.data)

	def __int__(self):
		""" On 'int(self)'."""
		return int(self.data)

	def __long__(self):
		""" On 'long(self)'."""
		return long(self.data)

	def __float__(self):
		""" On 'float(self)'."""
		return float(self.data)

	def __complex__(self):
		""" On 'complex(self)'."""
		return complex(self.data)

	def __nonzero__(self):
		""" On truth-value (or uses '__len__()' if defined)."""
		return self.data != 0

	def __coerce__(self, other):
		""" On mixed-type expression, 'coerce()'. Returns tuple of '(self, other)'
			converted to a common type."""
		return coerce(self.data, other)


class Integer(Number):
	""" Emulates a Python integer."""

	TYPE = type(1)

	def _reset(self):
		""" Initialize an integer."""
		self.data = 0

	def _convert(self, data):
		""" Convert data into an integer."""
		return int(data)

	def __lshift__(self, other):
		""" On 'self<<other'."""
		return self.data << other

	def __rlshift__(self, other):
		""" On 'other<<self'."""
		return other << self.data

	def __rshift__(self, other):
		""" On 'self>>other'."""
		return self.data >> other

	def __rrshift__(self, other):
		""" On 'other>>self'."""
		return other >> self.data

	def __and__(self, other):
		""" On 'self&other'."""
		return self.data & other

	def __rand__(self, other):
		""" On 'other&self'."""
		return other & self.data

	def __or__(self, other):
		""" On 'self|other'."""
		return self.data | other

	def __ror__(self, other):
		""" On 'other|self'."""
		return other | self.data

	def __xor__(self, other):
		""" On 'self^other'."""
		return self.data ^ other

	def __rxor__(self, other):
		""" On 'other%self'."""
		return other % self.data

	def __invert__(self):
		""" On '~self'."""
		return ~self.data

	def __oct__(self):
		""" On 'oct(self)'. Returns octal string representation."""
		return oct(self.data)

	def __hex__(self):
		""" On 'hex(self)'. Returns hexidecimal string representation."""
		return hex(self.data)


class Long(Integer):
	""" Emulates a Python long integer."""

	TYPE = type(1L)

	def _reset(self):
		""" Initialize an integer."""
		self.data = 0L

	def _convert(self, data):
		""" Convert data into an integer."""
		return long(data)


class Float(Number):
	""" Emulates a Python floating-point number."""

	TYPE = type(0.1)

	def _reset(self):
		""" Initialize a float."""
		self.data = 0.0

	def _convert(self, data):
		""" Convert data into a float."""
		return float(data)


class Complex(Number):
	""" Emulates a Python complex number."""

	TYPE = type(0+0j)

	def _reset(self):
		""" Initialize an integer."""
		self.data = 0+0j

	def _convert(self, data):
		""" Convert data into an integer."""
		return complex(data)

	def __getattr__(self, name):
		""" On 'self.real' & 'self.imag'."""
		if name == "real":
			return self.data.real
		elif name == "imag":
			return self.data.imag
		else:
			raise AttributeError(name)

	def conjugate(self):
		""" On 'self.conjugate()'."""
		return self.data.conjugate()


class Container(SuperType):
	""" Superclass for countable, indexable collection types ('Sequence', 'Mapping')."""

	def __len__(self):
		""" On 'len(self)', truth-value tests. Returns sequence or mapping
			collection size. Zero means false."""
		return len(self.data)

	def __getitem__(self, key):
		""" On 'self[key]', 'x in self', 'for x in self'. Implements all
			indexing-related operations. Membership and iteration ('in', 'for')
			repeatedly index from 0 until 'IndexError'."""
		return self.data[key]


class Sequence(Container, AddMulMixin):
	""" Superclass for classes which emulate sequences ('List', 'Tuple', 'String')."""

	def __getslice__(self, low, high):
		""" On 'self[low:high]'."""
		return self.data[low:high]


class String(Sequence, ModMixin):
	""" Emulates a Python string."""

	TYPE = type("")
	
	def _reset(self):
		""" Initialize an empty string."""
		self.data = ""

	def _convert(self, data):
		""" Convert data into a string."""
		return str(data)


class Tuple(Sequence):
	""" Emulates a Python tuple."""

	TYPE = type(())

	def _reset(self):
		""" Initialize an empty tuple."""
		self.data = ()

	def _convert(self, data):
		""" Non-tuples cannot be converted. Raise an exception."""
		raise TypeError("Non-tuples cannot be converted to a tuple.")


class MutableSequence(Sequence, MutableMixin):
	""" Superclass for classes which emulate mutable (modifyable in-place)
		sequences ('List')."""

	def __setslice__(self, low, high, seq):
		""" On 'self[low:high]=seq'."""
		self.data[low:high] = seq

	def __delslice__(self, low, high):
		""" On 'del self[low:high]'."""
		del self.data[low:high]

	def append(self, x):
		""" Inserts object 'x' at the end of 'self.data' in-place."""
		self.data.append(x)

	def count(self, x):
		""" Returns the number of occurrences of 'x' in 'self.data'."""
		return self.data.count(x)

	def extend(self, x):
		""" Concatenates sequence 'x' to the end of 'self' in-place 
			(like 'self=self+x')."""
		self.data.extend(x)

	def index(self, x):
		""" Returns the offset of the first occurrence of object 'x' in
			'self.data'; raises an exception if not found."""
		return self.data.index(x)

	def insert(self, i, x):
		""" Inserts object 'x' into 'self.data' at offset 'i' 
			(like 'self[i:i]=[x]')."""
		self.data.insert(i, x)

	def pop(self, i=-1):
		""" Returns and deletes the last item of 'self.data' (or item
			'self.data[i]' if 'i' given)."""
		return self.data.pop(i)

	def remove(self, x):
		""" Deletes the first occurrence of object 'x' from 'self.data'; 
			raise an exception if not found."""
		self.data.remove(x)

	def reverse(self):
		""" Reverses items in 'self.data' in-place."""
		self.data.reverse()

	def sort(self, func=None):
		"""
		Sorts 'self.data' in-place. Argument:
		- func : optional, default 'None' --
			- If 'func' not given, sorting will be in ascending
			  order.
			- If 'func' given, it will determine the sort order.
			  'func' must be a two-argument comparison function
			  which returns -1, 0, or 1, to mean before, same,
			  or after ordering."""
 		if func:
			self.data.sort(func)
		else:
			self.data.sort()


class List(MutableSequence):
	""" Emulates a Python list. When instantiating an object with data
		('List(data)'), you can force a copy with 'List(list(data))'."""

	TYPE = type([])

	def _reset(self):
		""" Initialize an empty list."""
		self.data = []

	def _convert(self, data):
		""" Convert data into a list."""
		return list(data)


class Mapping(Container):
	""" Superclass for classes which emulate mappings/hashes ('Dictionary')."""

	def has_key(self, key):
		""" Returns 1 (true) if 'self.data' has a key 'key', or 0 otherwise."""
		return self.data.has_key(key)

	def keys(self):
		""" Returns a new list holding all keys from 'self.data'."""
		return self.data.keys()

	def values(self):
		""" Returns a new list holding all values from 'self.data'."""
		return self.data.values()

	def items(self):
		""" Returns a new list of tuple pairs '(key, value)', one for each entry
			in 'self.data'."""
		return self.data.items()

	def clear(self):
		""" Removes all items from 'self.data'."""
		self.data.clear()

	def get(self, key, default=None):
		""" Similar to 'self[key]', but returns 'default' (or 'None') instead of
			raising an exception when 'key' is not found in 'self.data'."""
		return self.data.get(key, default)

	def copy(self):
		""" Returns a shallow (top-level) copy of 'self.data'."""
		return self.data.copy()

	def update(self, dict):
		""" Merges 'dict' into 'self.data' 
			(i.e., 'for (k,v) in dict.items(): self.data[k]=v')."""
		self.data.update(dict)


class Dictionary(Mapping, MutableMixin):
	""" Emulates a Python dictionary, a mutable mapping. When instantiating an
		object with data ('Dictionary(data)'), you can force a (shallow) copy
		with 'Dictionary(data.copy())'."""

	TYPE = type({})

	def _reset(self):
		""" Initialize an empty dictionary."""
		self.data = {}

	def _convert(self, data):
		""" Non-dictionaries cannot be converted. Raise an exception."""
		raise TypeError("Non-dictionaries cannot be converted to a dictionary.")


if __name__ == "__main__":
	print __doc__								# show module's documentation string



from distutils.command.build import build as DistutilsBuild
from setuptools import setup, Extension
import subprocess
import numpy

# For building Pachi as a library
class BuildLibPachi(DistutilsBuild):
    def run(self):
        try:
            subprocess.check_call("cd pachi_py; mkdir -p build && cd build && cmake ../pachi && make -j4", shell=True)
        except subprocess.CalledProcessError, e:
            print("Could not build pachi-py: %s" % e)
            raise
        DistutilsBuild.run(self)

# Cython recommands checking in the Cython-generated C files
# (cf. http://stackoverflow.com/a/19138055).
# After changing pachi_py/cypachi.pyx, recompile using:
#
#     $ cython --cplus pachi_py/cypachi.pyx
#
ext = Extension(
    name="pachi_py.cypachi",
    sources=["pachi_py/cypachi.cpp", "pachi_py/goutil.cpp"],
    language="c++",
    include_dirs=[numpy.get_include(), "pachi_py/pachi"],
    libraries=["pachi"],
    library_dirs=["pachi_py/build/lib"], # this is the output dir of BuildLibPachi
    extra_compile_args=["-std=c++11"],
)

setup(name='pachi-py',
      version='0.0.9',
      description='Python bindings to Pachi',
      url='https://github.com/rl-gym/pachi-py',
      author='OpenAI',
      author_email='info@openai.com',
      packages=['pachi_py'],
      cmdclass={'build': BuildLibPachi},
      install_requires=['numpy'],
      tests_require=['nose2'],
      classifiers=['License :: OSI Approved :: GNU General Public License v2 (GPLv2)'],
      ext_modules=[ext],
      include_package_data=True,
)
